import 'package:flutter/material.dart';
import 'package:bmi_global/utils/tiles.dart';

class _TitleText extends StatelessWidget {
  const _TitleText(this.data, {Key? key}) : super(key: key);
  final String data;

  @override
  Widget build(BuildContext context) {
    return Text(
      data,
      style: TextStyle(color: Theme.of(context).colorScheme.tertiary),
    );
  }
}

class DisplayCell {
  final String val;
  final void Function()? onTap;
  DisplayCell(this.val, [this.onTap]);
}

class DisplayRow {
  final String fixedCell;
  final Iterable<DisplayCell> cells;

  DisplayRow.str({required this.fixedCell, required Iterable<String> cells})
      : cells = cells.map((e) => DisplayCell(e));
  DisplayRow({required this.fixedCell, required this.cells});
}

class DisplaySelectRow<T> {
  final Set<T> selected;
  final void Function(T, bool) onSelectChange;
  DisplaySelectRow({required this.selected, required this.onSelectChange});
}

class DisplayTable<T> extends StatelessWidget {
  const DisplayTable({
    Key? key,
    required this.fixColumn,
    required this.column,
    required this.values,
    required this.rowBuilder,
    this.trailing,
    this.displaySelectRow,
    this.onRowTap,
  }) : super(key: key);

  final String fixColumn;
  final Iterable<String> column;
  final Iterable<T> values;
  final DisplayRow Function(T val) rowBuilder;
  final DisplayRow? trailing;
  final DisplaySelectRow<T>? displaySelectRow;
  final void Function(T val)? onRowTap;

  @override
  Widget build(BuildContext context) {
    if (column.isEmpty || values.isEmpty) {
      return const ErrorTile("* No Data to show");
    }
    final col1 = [DataColumn(label: _TitleText(fixColumn))];
    final col2 = column.map((e) => DataColumn(label: _TitleText(e))).toList();
    final row1 = <DataRow>[];
    final row2 = <DataRow>[];
    for (var item in values) {
      final row = rowBuilder(item);
      row1.add(
        DataRow(
          cells: [
            DataCell(
              Text(row.fixedCell),
              onTap: onRowTap == null ? null : () => onRowTap?.call(item),
            )
          ],
          selected: displaySelectRow?.selected.contains(item) ?? false,
          onSelectChanged: (b) {
            displaySelectRow?.onSelectChange(item, b ?? false);
          },
        ),
      );
      row2.add(
        DataRow(
          cells: row.cells
              .map(
                (e) => DataCell(
                  Text(e.val),
                  onTap:
                      onRowTap == null ? e.onTap : () => onRowTap?.call(item),
                ),
              )
              .toList(),
        ),
      );
    }
    if (trailing != null) {
      row1.add(DataRow(cells: [DataCell(_TitleText(trailing!.fixedCell))]));
      row2.add(
        DataRow(
          cells: trailing!.cells
              .map((e) => DataCell(Text(e.val), onTap: e.onTap))
              .toList(),
        ),
      );
    }
    return Row(
      children: [
        Material(
          elevation: 15,
          child: DataTable(
            columns: col1,
            rows: row1,
            showCheckboxColumn: displaySelectRow != null,
          ),
        ),
        Expanded(
          child: SingleChildScrollView(
            scrollDirection: Axis.horizontal,
            child: DataTable(
              columns: col2,
              rows: row2,
            ),
          ),
        ),
      ],
    );
  }
}
